@@ -20,7 +20,7 @@ fi
 name="yaws"
 rcvar=$name
 yaws_command="%prefix%bin/${name}"
-required_files="%etcdir%yaws.conf"
+required_files="%etcdir%yaws/yaws.conf"
 
 start_cmd="yaws_start"
 stop_cmd="yaws_stop"
