@@ -23,6 +23,7 @@
 -module(haxe).
 -export([encode/1, encode/2, decode_string/1, decode_string/2,
          decode/1, decode/2, decode_next/1]).
+-export([get_left_over_chars/1]).
 -export([is_obj/1, obj_new/0, obj_fetch/2, obj_find/2, obj_is_key/2]).
 -export([test/0]).
 -export([obj_store/3, obj_from_list/1, obj_fold/3]).
@@ -34,21 +35,23 @@
 
 %%% This module translates haXe types into the following Erlang types:
 %%%
-%%%        haXe                         Erlang
-%%%        ----                         ------
-%%%        int, float                 number
-%%%        string (ascii, utf8)         string
-%%%        array                         {array, ElementList}
-%%%        true, false, null         atoms 't', 'f', and 'n'
-%%%     Number.NaN               atom 'nan'
-%%%     Number.NEGATIVE_INFINITY atom 'neg_infinity'
-%%%     Number.POSITIVE_INFINITY atom 'infinity'
-%%%        object                         tagged proplist with string (or atom) keys
-%%%                              (e.g. {struct, [{foo, "bar"}, {baz, 4}]} )
-%%%     class object             NOT SUPPORTED
-%%%     enum                     NOT SUPPORTED
-%%%     reference                {ref, Idx}, Idx -> int()
-%%%     exception                {exception, Obj}, Obj is any of the haXe types above
+%%%     haXe                         Erlang
+%%%     ----                         ------
+%%%     int, float                   number
+%%%     string (ascii, utf8)         string
+%%%     string (base64)              binary
+%%%     array                        {array, ElementList}
+%%%     true, false, null            atoms 't', 'f', and 'n'
+%%%     Number.NaN                   atom 'nan'
+%%%     Number.NEGATIVE_INFINITY     atom 'neg_infinity'
+%%%     Number.POSITIVE_INFINITY     atom 'infinity'
+%%%     object                       tagged proplist with string (or atom) keys
+%%%                                  (e.g. {struct, [{foo, "bar"}, {baz, 4}]} )
+%%%     class object                 NOT SUPPORTED
+%%%     enum                         NOT SUPPORTED
+%%%     reference                    {ref, Idx}, Idx -> int()
+%%%     exception                    {exception, Obj}, Obj is any
+%%%                                  of the haXe types above
 %%%
 %%% Classs objects and Enums are currently not supported
 %%% because simulating such types in Erlang is quite cumbersome, plus
@@ -76,29 +79,56 @@
 encode({ref, _Idx} = Ref) -> encode_basic(Ref);
 
 
-encode(Obj) -> encode(Obj, true).
+encode(Obj) -> encode(Obj, true, 2).
 
-%% The second parameter indicates whether the cache should
+%% The second parameter is a list of options.
+%% The following options are available:
+%%    {use_cache, boolean()}      -- default = true
+%%    {haxe_version, Vsn}         -- default = 2, can be 1 or 2
+%% 
+%% The haxe_version is a rough indication of what haXe version we
+%% should encode to. This does not really capture the different
+%% encodings used in the different haXe 1.x versions, see the
+%% token_identifier function in this module for details.
+%%
+%% There second parameter can also be a boolean for backwards compatibility,
+%% in which case the latest haxe_version (2) is assumed.
+encode(Obj, Opts) when is_list(Opts) ->
+    UseCache = proplists:get_value(use_cache, Opts, true),
+    Vsn = proplists:get_value(haxe_version, Opts, 2),
+    encode(Obj, UseCache, Vsn);
+% function clauses for backwards compatibility:
+encode(Obj, true=UseCache)  -> encode(Obj, UseCache, 2);
+encode(Obj, false=UseCache) -> encode(Obj, UseCache, 2).
+
+
+%% The second parameter, if a boolean, indicates whether the cache should
 %% be used. By default, it's turned on (this follows the default configuration
 %% in haXe).
-encode(Obj, true) ->
-    {Result, _Cache} = encode(Obj, dict:new()),
+encode(Obj, true, Vsn) ->
+    {Result, _Cache} = encode(Obj, dict:new(), Vsn),
     Result;
-encode(Obj, false) ->
-    {Result, _Cache} = encode(Obj, undefined),
+encode(Obj, false, Vsn) ->
+    {Result, _Cache} = encode(Obj, undefined, Vsn),
     Result;
-encode(L, Cache) when is_list(L) ->
+encode(L, Cache, _Vsn=1) when is_list(L) ->
     case is_string(L) of
-        yes -> encode_string(L, $s, Cache);
-        unicode -> encode_string(xmerl_ucs:to_utf8(L), $j, Cache);
+        yes -> encode_string1(L, $s, Cache);
+        unicode -> encode_string1(xmerl_ucs:to_utf8(L), $j, Cache);
         no -> exit({error, {haxe_encode, {not_string, L}}})
     end;
-encode({array, Props}, Cache) -> encode_array(Props, Cache);
-encode({struct, Props}, Cache) -> encode_object(Props, Cache);
-encode({exception, E}, Cache) -> 
-    {Result, Cache2} = encode(E, Cache),
+encode(L, Cache, _Vsn=2) when is_list(L) ->
+    case is_string(L) of
+        no -> exit({error, {haxe_encode, {not_string, L}}});
+        _  -> L2 = yaws_api:url_encode(xmerl_ucs:to_utf8(L)),
+              encode_string1(L2, $y, Cache)
+    end;
+encode({array, Props}, Cache, Vsn) -> encode_array(Props, Cache, Vsn);
+encode({struct, Props}, Cache, Vsn) -> encode_object(Props, Cache, Vsn);
+encode({exception, E}, Cache, Vsn) -> 
+    {Result, Cache2} = encode(E, Cache, Vsn),
     {[$x | Result], Cache2};
-encode(Term, Cache) ->
+encode(Term, Cache, _Vsn) ->
     case encode_basic(Term) of
         {error, _Err} = Err ->
             exit(Err);
@@ -119,6 +149,8 @@ encode_basic(Term) ->
         0 -> "z";
         I when is_integer(I) -> [$i | integer_to_list(I)];
         F when is_float(F) -> [$d | io_lib:format("~g", [F])];
+        B when is_binary(B) -> S = binary_to_list(base64:encode(B)),
+                               [$s, integer_to_list(length(S)), $: | S];
         _ -> {error, {bad_term, Term}}
     end.
 
@@ -138,40 +170,31 @@ find_ref(Str, CacheDict) ->
     end.
 
 %% Encode an Erlang string to haXe.
-encode_string(Str, Cache) ->
-    encode_string(Str, $s, Cache).
+encode_string(Str, Cache, 1=_Vsn) -> encode_string1(Str, $s, Cache);
+encode_string(Str, Cache, 2=_Vsn) -> encode_string1(Str, $y, Cache).
 
-encode_string(Str, FirstChar, Cache) ->
+encode_string1(Str, FirstChar, Cache) ->
     case find_ref(Str, Cache) of
         false ->
-            case encode_string2(Str, FirstChar) of
-                {ok, Result} ->
-                    NewCache =
-                        case Cache of
-                            undefined ->
-                                undefined;
-                            _ ->
-                                dict:store(Str, dict:size(Cache), Cache)
-                        end,
-                    {Result, NewCache};
-                Err ->
-                    Err
-            end;
+            {ok, Result} = encode_string2(Str, FirstChar),
+            NewCache =
+                case Cache of
+                    undefined ->
+                        undefined;
+                    _ ->
+                        dict:store(Str, dict:size(Cache), Cache)
+                end,
+            {Result, NewCache};
         {true, Idx} ->
             {[$R | integer_to_list(Idx)], Cache}
     end.
+encode_string2(S, $y) ->
+    {ok, [$y, integer_to_list(length(S)), $: | S]};
 encode_string2(S, FirstChar) ->
     encode_string2(S, FirstChar, []).
 encode_string2([], FirstChar, Acc) ->
     Str = lists:reverse(Acc),
-    Len = integer_to_list(
-            length(
-              case FirstChar of
-                  $j ->
-                      xmerl_ucs:from_utf8(Str);
-                  _ ->
-                      Str
-              end)),
+    Len = integer_to_list(length(Str)),
     {ok, [FirstChar, Len, $: | Str]};
 encode_string2([C | Cs], FirstChar, Acc) ->
     case C of
@@ -182,20 +205,20 @@ encode_string2([C | Cs], FirstChar, Acc) ->
         _ -> exit({error, {haxe_encode, {bad_char, C}}})
     end.
 
-encode_object(Props, Cache) ->
-    {Result, Cache2} = encode_object_rest(Props, Cache),
+encode_object(Props, Cache, Vsn) ->
+    {Result, Cache2} = encode_object_rest(Props, Cache, Vsn),
     {[$o | Result], Cache2}.
-encode_object_rest(Props, Cache) ->
+encode_object_rest(Props, Cache, Vsn) ->
     {EncodedProps, Cache1} =
         lists:foldl(
           fun({Key, Value}, {Acc, Cache2}) ->
                   {EncodedKey, Cache3} =
                       case Key of
-                          L when is_list(L) -> encode_string(L, Cache2);
-                          A when is_atom(A) -> encode_string(atom_to_list(A), Cache2);
+                          L when is_list(L) -> encode_string(L, Cache2, Vsn);
+                          A when is_atom(A) -> encode_string(atom_to_list(A), Cache2, Vsn);
                           _ -> exit({error, {haxe_encode, {bad_key, Key}}})
                       end,
-                  {EncodedVal, Cache4} = encode(Value, Cache3),
+                  {EncodedVal, Cache4} = encode(Value, Cache3, Vsn),
                   case Acc of
                       [] -> {[[EncodedKey, EncodedVal]], Cache4};                                        _  -> {[[EncodedKey, EncodedVal] | Acc], Cache4}
                   end
@@ -207,17 +230,17 @@ encode_object_rest(Props, Cache) ->
     Result = [lists:reverse(EncodedProps), $g],
     {Result, Cache1}.
 
-encode_array(Props, Cache) ->
+encode_array(Props, Cache, Vsn) ->
     {NullCount, Arr, Cache1} = lists:foldl(
           fun
               (Elem, {NullCount, Arr, Cache2}) when Elem == null;
                                                     Elem == undefined ->
                   {NullCount+1, Arr, Cache2};
               (Elem, {0, Arr, Cache2}) ->
-                  {Encoded, Cache3} = encode(Elem, Cache2),
+                  {Encoded, Cache3} = encode(Elem, Cache2, Vsn),
                   {0, [Encoded | Arr], Cache3};
               (Elem, {NullCount, Arr, Cache2}) ->
-                  {Encoded, Cache3}  = encode(Elem, Cache2),
+                  {Encoded, Cache3}  = encode(Elem, Cache2, Vsn),
                   {0, [Encoded, encode_nulls(NullCount) | Arr], Cache3}
           end,
           {0, [$a], Cache}, Props),
@@ -242,11 +265,10 @@ encode_nulls(Num) -> [$u | integer_to_list(Num)].
 %%% it can delimit a number, and otherwise causes a match failure.
 %%%
 
-token([]) -> {more, []};
-token(eof) -> {done, eof, []};
-
-token([C | Rest]) ->
-    case token_identifier(C) of
+token_vsn1([]) -> {more, []};
+token_vsn1(eof) -> {done, eof, []};
+token_vsn1([C | Rest]) ->
+    case token_identifier(1, C) of
         int -> scan_int(Rest);
         float -> scan_float(Rest);
         string -> scan_string(Rest);
@@ -255,12 +277,31 @@ token([C | Rest]) ->
         Token -> {done, {ok, Token}, Rest}
     end.
 
-token_identifier(C) ->
+token_vsn2([]) -> {more, []};
+token_vsn2(eof) -> {done, eof, []};
+token_vsn2([C | Rest]) ->
+    case token_identifier(2, C) of
+        int -> scan_int(Rest);
+        float -> scan_float(Rest);
+        string -> scan_string(Rest);
+        url_encoded_string -> scan_url_encoded_string(Rest);
+        base64_bytes -> scan_base64_bytes(Rest);
+        error -> {done, {error, invalid_token}, Rest};
+        Token -> {done, {ok, Token}, Rest}
+    end.
+
+token_identifier(Vsn, C) ->
     case C of
         $i -> int;
         $d -> float;
-        $s -> string;
-        $j -> utf8;
+        $s -> case Vsn of
+                  1 -> string;
+                  2 -> base64_bytes %% redefined to bytes (base64) in haXe 2.0
+              end;
+        $j -> case Vsn of
+                  1 -> utf8;
+                  2 -> enum  %% redefined from utf8 to enum in haXe 1.16
+              end;
         $n -> null;
         $t -> true;
         $f -> false;   
@@ -276,35 +317,50 @@ token_identifier(C) ->
         $r -> ref;
         $R -> str_ref;
         $x -> exception;
+        $y -> case Vsn of
+                  1 -> error;
+                  2 -> url_encoded_string %% utf8; added/defined in haXe 1.11
+              end;
 
         %% this token is only valid as an element
         %% of an array
         $u -> null_seq;
         _ -> error
     end.
-              
+
 scan_utf8(Chars) ->
-    scan_chars(Chars, true).
+    scan_chars(Chars, utf8).
 
 scan_string(Chars) ->
-    scan_chars(Chars, false).
+    scan_chars(Chars, string).
 
-scan_chars(Chars, IsUtf8) ->
+scan_url_encoded_string(Chars) ->
+    scan_chars(Chars, url_encoded).
+
+scan_base64_bytes(Chars) ->
+    scan_chars(Chars, base64_bytes).
+
+scan_chars(Chars, Type) ->
     case scan_int(Chars) of
         {done, {ok, _NumBytes}, []} ->
             {more, Chars};
         {done, {ok, _NumBytes}, [C | _Rest]} when C /= $: ->
             {done, {error, bad_char, C}, Chars};
         {done, {ok, NumBytes}, [_ | Rest]} when length(Rest) >= NumBytes ->
-            {Str, NewLength} =
-                case IsUtf8 of
-                    true ->
-                        NewStr = xmerl_ucs:to_utf8(lists:sublist(Rest, NumBytes)),
-                        {NewStr, length(NewStr)};
-                    false ->
-                        {Rest, NumBytes}
-                end,
-            scan_chars(Str, [], NewLength);
+            case Type of
+                utf8 ->
+                    NewStr = xmerl_ucs:from_utf8(lists:sublist(Rest, NumBytes)),
+                    scan_chars(NewStr, [], length(NewStr));
+                string ->
+                    scan_chars(Rest, [], NumBytes);
+                url_encoded ->
+                    {S, Rest2} = split_str_after(Rest, NumBytes),
+                    S2 = xmerl_ucs:from_utf8(yaws_api:url_decode(S)),
+                    {done, {ok, S2}, Rest2};
+                base64_bytes ->
+                    {S, Rest2} = split_str_after(Rest, NumBytes),
+                    {done, {ok, base64:decode(S)}, Rest2}
+            end;
         {done, {ok, _NumBytes}, _Rest} ->
             {more, Chars};
         Other ->
@@ -330,6 +386,10 @@ esc_to_char(C) ->
         $\\ -> $\\
     end.
 
+split_str_after(S, AfterNumBytes) ->
+    {string:substr(S, 1, AfterNumBytes), string:substr(S, AfterNumBytes + 1)}.
+
+
 scan_float(Chars) ->
     scan_number(Chars, float).
 
@@ -423,46 +483,89 @@ scan_exponent(Rest, Es, R, _X) ->
 %% Return the first haXe value decoded from the input string.
 %% The string must contain at least one complete haXe value.
 %%
-%% The second (optional) parameter indicates if caching should
-%% be turned on. Caching is on by default, but it can be turned
+%% The second (optional) parameter can be either:
+%% - a list of options:
+%%     {use_cache, boolean()}   -- default = true
+%%     {haxe_version, Vsn}      -- default = 2, can be 1 or 2
+%% - a continuation, if {more,Continuation} was previously
+%%   returned and you now have more characters
+%% - a boolean specifying whether to use the cache,
+%%   for backwards compatibility
+%%
+%% The haxe_version is a rough indication of what haXe version we
+%% should expect to decode. This does not really capture the different
+%% encodings used in the different haXe 1.x versions, see the
+%% token_identifier function in this module for details.
+%%
+%% There second parameter can also be a boolean for backwards compatibility,
+%% in which case the latest haxe_version (2) is assumed.
+%%
+%% Caching is on by default, but it can be turned
 %% off for performance and lower memory consumption depending
 %% on the content of the messages.
 decode_string(CharList) ->
-    decode_string(CharList, true).
-decode_string(CharList, UseCache) ->
-    {done, V, _} = decode(CharList ++ [eof], UseCache),
+    decode_string(CharList, [{use_cache, true}]).
+decode_string(CharList, Opts) ->
+    {done, V, _} = decode(CharList ++ [eof], Opts),
     V.
 
 %% Attempt to decode a haXe value from the input string
 %% and continuation, using an empty initial continuation.
-%% Return {done, Result, {LeftoverChars, Cache}} if a value is recognized,
+%% Return {done, Result, Continuation} if a value is recognized,
 %% or {more, Continuation} if more input characters are needed.
 %% The Result can be {ok, Value}, eof, or {error, Reason}.
 %% The Continuation is then fed as an argument to decode/2 when
 %% more input is available.
 %% Use the atom 'eof' instead of a char list to signal
 %% a true end to the input, and may flush a final number.
+%%
+%% Use the get_left_over_chars/1 function to retrieve any non-consumed
+%% characters in the continuation.
 
 decode(CharList) ->
-    decode(CharList, true).
+    decode_o(CharList, [{use_cache, true}]).
 
-%% Note: our Cache for decoding is the tuple
-%% {ObjCache::list(), StringCache::dict()}
-decode(CharList, true) ->
-    decode2(CharList, {[], dict:new()});
-decode(CharList, false) ->
-    decode2(CharList, undefined).
+-record(cont,{chars,
+              cache,
+              tokenizer}).
 
-decode2(CharList, Cache) ->
-    decode3({{[], Cache}, fun first_cont_fun/2}, CharList).
+decode(CharList, Opts) when is_list(Opts) ->
+    decode_o(CharList, Opts);
+decode(CharList, Cont) when is_record(Cont, cont) -> %% Continuation case
+    #cont{chars = Chars, cache = Cache, tokenizer = Tokenizer} = Cont,
+    decode2(Chars ++ CharList, Cache, Tokenizer);
+decode(CharList, true) ->  %% function clause for backwards compatibility
+    decode_o(CharList, [{use_cache, true}]);
+decode(CharList, false) -> %% function clause for backwards compatibility
+    decode_o(CharList, [{use_cache, false}]).
+    
+decode_o(CharList, Opts) ->
+    %% Note: our Cache for decoding is the tuple
+    %% {ObjCache::list(), StringCache::dict()}
+    Cache = case proplists:get_value(use_cache, Opts, true) of
+                true  -> {[], dict:new()};
+                false -> undefined
+            end,
+    Tokenizer = case proplists:get_value(haxe_version, Opts, 2) of
+                    2 -> fun token_vsn2/1;
+                    1 -> fun token_vsn1/1 %% haxe uses "old_serializer"
+                end,
+    decode2(CharList, Cache, Tokenizer).
 
-decode3({{[],Cache}, Kv}, CharList) ->
-    get_token({CharList, Cache}, Kv);
-decode3({{OldChars, Cache}, Kv}, CharList) ->
-    get_token({OldChars ++ CharList, Cache}, Kv).
 
-decode_next({OldChars, Cache}) ->
-    decode3({{[], Cache}, fun first_cont_fun/2}, OldChars).
+decode2(CharList, Cache, Tokenizer) ->
+    Cont = #cont{chars = CharList,
+                 cache = Cache,
+                 tokenizer = Tokenizer},
+    get_token(Cont, fun first_cont_fun/2).
+
+
+decode_next(Cont) ->
+    get_token(Cont, fun first_cont_fun/2).
+
+get_left_over_chars(#cont{chars=Cs}) ->
+    Cs.
+
 
 first_cont_fun(eof, Cs) -> {done, eof, Cs};
 first_cont_fun(T, Cs) ->
@@ -471,21 +574,22 @@ first_cont_fun(T, Cs) ->
                         {done, {ok, V}, C2}
                 end).
 
-%% Continuation Kt must accept (TokenOrEof, {Chars, Cache})
+%% Continuation Kt must accept (TokenOrEof, {Chars, Cache, Tokenizer})
 
-get_token({Chars, Cache}, Kt) ->
-    case token(Chars) of
-        {done, {ok, T}, Rest} -> Kt(T, {Rest, Cache});
-        {done, eof, Rest} -> Kt(eof, {Rest, Cache});
-        {done, {error, Reason}, Rest} -> {done, {error, Reason}, {Rest, Cache}};
-        {more, X} -> {more, {X, Kt}}
+get_token(C = #cont{chars=Chars, tokenizer=Tokenizer}, Kt) ->
+    case Tokenizer(Chars) of
+        {done, {ok, T}, Rest} -> Kt(T, C#cont{chars=Rest});
+        {done, eof, Rest} -> Kt(eof, C#cont{chars=Rest});
+        {done, {error, Reason}, Rest} -> {done, {error, Reason},
+                                          C#cont{chars=Rest}};
+        {more, X} -> {more, C#cont{chars=X}}
     end.
 
-%% Continuation Kv must accept (Value, {Chars, Cache})
+%% Continuation Kv must accept (Value, {Chars, Cache, Tokenizer})
 
 parse_value(Token, C, Kv) ->
     parse_value(Token, C, Kv, false).
-parse_value(Token, {Chars, Cache} = C, Kv, AcceptNullSeq)->
+parse_value(Token, #cont{cache=Cache} = C, Kv, AcceptNullSeq)->
     case Token of
         eof -> {done, {error, premature_eof}, C};
         T when T == null; T == true; T == false; T == nan;
@@ -499,12 +603,10 @@ parse_value(Token, {Chars, Cache} = C, Kv, AcceptNullSeq)->
         ref -> parse_ref(C, Kv);
         str_ref -> parse_ref(C, Kv, true);
         null_seq when AcceptNullSeq -> parse_null_seq(C, Kv);
-        Str when is_list(Str) ->
-            Kv(Str,
-               {Chars,
-                put_str(Str, Cache)});
+        Str when is_list(Str) -> Kv(Str, C#cont{cache=put_str(Str, Cache)});
         Num when is_number(Num) -> Kv(Num, C);
-        _ -> {done, {error, syntax_error}, C}
+        B   when is_binary(B) -> Kv(B, C#cont{cache=put_str(B, Cache)});
+        X -> {done, {error, syntax_error,X}, C}
     end.
 
 parse_class(Chars, _Kv) ->
@@ -515,18 +617,18 @@ parse_object(Chars, Kv) ->
 
 parse_object(C, Kv, Obj) ->
     get_token(C,
-              fun(T, {Chars, Cache}) ->
+              fun(T, C2=#cont{cache=Cache}) ->
                       case T of
                           obj_end ->
                               NewCache =
                                   put_obj(Obj, Cache),
-                              Kv(Obj, {Chars, NewCache});
+                              Kv(Obj, C2#cont{cache=NewCache});
                           _ ->
                               NewCache =
                                   put_obj(placeholder, Cache),
                               parse_object_field(
                                 Obj, T,
-                                {Chars, NewCache}, Cache, Kv)
+                                C2#cont{cache=NewCache}, Cache, Kv)
                       end
               end).
 
@@ -553,14 +655,14 @@ parse_object_field(Obj, RefType, C, OrigCache, Kv)
              RefType == str_ref);
 
 %% if the key is a string, we put it in the cache and continue
-parse_object_field(Obj, Field, {Chars, Cache}, OrigCache, Kv)
+parse_object_field(Obj, Field, C=#cont{cache=Cache}, OrigCache, Kv)
   when is_list(Field) ->
     NewCache = put_str(Field, Cache),
-    parse_object_val(Obj, Field, {Chars, NewCache}, OrigCache, Kv).
+    parse_object_val(Obj, Field, C#cont{cache=NewCache}, OrigCache, Kv).
 
-parse_object_val(Obj, Field, {Chars, Cache}, OrigCache, Kv)
+parse_object_val(Obj, Field, C, OrigCache, Kv)
   when is_list(Field) ->
-    get_token({Chars, Cache},
+    get_token(C,
               fun(T, C2) ->
                       parse_value
                         (T, C2,
@@ -575,11 +677,11 @@ parse_object_val(_Obj, Field, C, _OrigCache, _Kv) ->
 parse_object_next({struct, Props} = Obj, C, OrigCache, Kv) ->
     get_token(C,
               fun
-                  (obj_end, {Chars, Cache}) ->
+                  (obj_end, C1 = #cont{cache=Cache}) ->
                       Obj1 = {struct, lists:reverse(Props)},
                       Cache1 = append_new_elems(
                                     Cache, Obj1, OrigCache),
-                      Kv(Obj1, {Chars, Cache1});
+                      Kv(Obj1, C1#cont{cache=Cache1});
                   (eof, C1) ->
                       {done, {error, premature_eof}, C1};
                   (T, C1) ->                
@@ -597,7 +699,7 @@ append_new_elems({TempObjCache, NewStrCache}, Obj,
     {NewElts ++ ObjCache1, NewStrCache}.
 
 
-parse_array({Chars, Cache}, Kv) ->
+parse_array(C = #cont{cache=Cache}, Kv) ->
 
     %% We need to put a temporary placeholder in the cache
     %% to comply with haXe's indexing scheme, which assumes
@@ -608,18 +710,18 @@ parse_array({Chars, Cache}, Kv) ->
     %%
     %% If it sounds backwards, well, it is! :)
 
-    parse_array([], {Chars, put_obj(placeholder, Cache)},
+    parse_array([], C#cont{cache=put_obj(placeholder, Cache)},
                 Cache, Kv).
 
 parse_array(Elems, C, OrigCache, Kv) ->
     get_token(C,
               fun
                   (eof, C1) -> {done, {error, premature_eof}, C1};
-                  (array_end, {Chars1, Cache1}) -> 
+                  (array_end, C1=#cont{cache=Cache1}) -> 
                       Arr = {array, lists:reverse(Elems)},
                       Cache2 = append_new_elems(
                                  Cache1, Arr, OrigCache),
-                      Kv(Arr, {Chars1, Cache2});
+                      Kv(Arr, C1#cont{cache=Cache2});
                   (T, C1) ->
                       parse_array_tok(Elems, T, C1, OrigCache, Kv) 
               end).
@@ -654,18 +756,18 @@ parse_exception(Cont, Kv) ->
 parse_ref(Cont, Kv) ->
     parse_ref(Cont, Kv, false).
 
-parse_ref({Chars, Cache}, Kv, IsStrRef) ->
+parse_ref(C=#cont{chars=Chars}, Kv, IsStrRef) ->
     case scan_int(Chars) of
         {done, {ok, Idx}, Chars1} ->
-            parse_ref2(IsStrRef, {Chars1, Cache}, Idx, Kv);
+            parse_ref2(IsStrRef, C#cont{chars=Chars1}, Idx, Kv);
         Other ->
             Other
     end.
 
-parse_ref2(_IsStrRef, {_Chars, undefined} = Cont, _Idx, _Kv) ->
+parse_ref2(_IsStrRef, #cont{cache=undefined} = Cont, _Idx, _Kv) ->
     {done, {error, references_disabled}, Cont};
 
-parse_ref2(false, {_Chars, {ObjCache, _StrCache}} = Cont, Idx, Kv) ->
+parse_ref2(false, #cont{cache={ObjCache, _StrCache}} = Cont, Idx, Kv) ->
     if Idx > length(ObjCache) ->
             {done, {error, {ref_idx_out_of_bounds, Idx}}, Cont};
        true ->
@@ -678,7 +780,7 @@ parse_ref2(false, {_Chars, {ObjCache, _StrCache}} = Cont, Idx, Kv) ->
             end
     end;
 
-parse_ref2(true, {_Chars, {_ObjCache, StrCache}} = Cont, Idx, Kv) ->
+parse_ref2(true, #cont{cache={_ObjCache, StrCache}} = Cont, Idx, Kv) ->
     Cond = Idx + 1 > dict:size(StrCache),
     if Cond ->
             {done, {error, {str_ref_idx_out_of_bounds, Idx}}, Cont};
@@ -689,10 +791,11 @@ parse_ref2(true, {_Chars, {_ObjCache, StrCache}} = Cont, Idx, Kv) ->
     
 
 
-parse_null_seq({Chars, Cache}, Kv) ->
+parse_null_seq(Cont = #cont{chars=Chars}, Kv) ->
     case scan_int(Chars) of
         {done, {ok, Num}, C1} when Num > 0 ->
-            Kv({null_seq, lists:duplicate(Num, null)}, {C1, Cache});
+            Cont1 = Cont#cont{chars=C1},
+            Kv({null_seq, lists:duplicate(Num, null)}, Cont1);
         Other ->
             Other
     end.
@@ -767,7 +870,7 @@ test() ->
     Tests = [
              {1, "i1"},
              {1.1, "d1.10000"},
-             {"foo", "s3:foo"},
+             {"foo", "y3:foo"},
              %% todo test utf8
              {null, "n"},
              {true, "t"},
@@ -781,21 +884,38 @@ test() ->
              {{array, [null, null]}, "au2h"},
              {{array, [3, 4, null, null, null, 5, null]}, "ai3i4u3i5nh"},
              {{struct, [{foo, "bar"}, {baz, "boing"}]},
-              "os3:foos3:bars3:bazs5:boingg"},
-             {{array, ["foo", "bar", "foo", {struct, [{bar, "baz"}, {foo, 123}]}]}, "as3:foos3:bars3:fooos3:bars3:bazs3:fooi123gh"},
-             {{exception, "bad"}, "xs3:bad"}
+              "oy3:fooy3:bary3:bazy5:boingg"},
+             {{array, ["foo", "bar", "foo", {struct, [{bar, "baz"}, {foo, 123}]}]}, "ay3:fooy3:bary3:foooy3:bary3:bazy3:fooi123gh"},
+             {{exception, "bad"}, "xy3:bad"}
             ],
 
     {Passed, Failed} = run_tests(Tests, false),
 
     Tests1 = [
-              {{array, ["foo", "bar", "foo"]}, "as3:foos3:barR0h"},
-              {{struct, [{foo, "bar"}, {bar, "foo"}]}, "os3:foos3:barR1R0g"},
-              {{array, ["foo", "bar", "foo", {struct, [{bar, "baz"}, {foo, 123}]}]}, "as3:foos3:barR0oR1s3:bazR0i123gh"}
+              {{array, ["foo", "bar", "foo"]}, "ay3:fooy3:barR0h"},
+              {{struct, [{foo, "bar"}, {bar, "foo"}]}, "oy3:fooy3:barR1R0g"},
+              {{array, ["foo", "bar", "foo", {struct, [{bar, "baz"}, {foo, 123}]}]}, "ay3:fooy3:barR0oR1y3:bazR0i123gh"}
              ],
     {Passed1, Failed1} = run_tests(Tests1, true),
+
+    Tests2 = [{"abc"++[16#e5,16#e4,16#f6],   %% a-ring,a-diaeresis,o-diaeresis
+               "y21:abc%C3%A5%C3%A4%C3%B6"}, %% url-encoded utf8
+              {<<"abc">>,
+               "s4:YWJj"}],
+    {Passed2, Failed2} = run_tests(Tests2, [{haxe_version,2}]),
+
+    %% version1 encode/decode (simple utf8)
+    %% This is mostly based on looking at the code of the haxe
+    %% Serializer.hx and Unserializer.hx (as of cvs-tag v1-10)
+    Tests3 = [{"abc\n"++[16#1e5,16#1e4,16#1f6],
+	       "j11:abc\\n"++[16#c7,16#a5,16#c7,16#a4,16#c7,16#b6]},
+              {"abc",
+               "s3:abc"}],
+    {Passed3, Failed3} = run_tests(Tests3, [{haxe_version,1}]),
+
     io:format("passed: ~w, failed: ~w\n",
-              [Passed + Passed1, Failed + Failed1]).
+              [Passed + Passed1 + Passed2 + Passed3,
+               Failed + Failed1 + Failed2 + Failed3]).
 
 run_tests(Tests, EnableReferences) ->
     lists:foldl(
@@ -810,8 +930,9 @@ run_tests(Tests, EnableReferences) ->
                       end,
               Agg1 = Check(Str, Encoded, Agg),
               Agg2 = Check(Term, Decoded, Agg1),
-              io:format("~s == ~s\n~w\n~w == ~w\n~w\n\n", [Str, Encoded, Str == Encoded,
-                                                           Term, Decoded, Term == Decoded]),
+              io:format("~s == ~s\n~w\n~w == ~w\n~w\n\n",
+                        [Str, Encoded, Str == Encoded,
+                         Term, Decoded, Term == Decoded]),
               Agg2
       end,
       {0, 0}, Tests).
