@@ -26,7 +26,8 @@
          htmlize/1, htmlize_char/1, f/2, fl/1]).
 -export([find_cookie_val/2, secs/0, 
          url_decode/1, url_decode_q_split/1,
-         url_encode/1, parse_url/1, parse_url/2, format_url/1]).
+         url_encode/1, parse_url/1, parse_url/2, format_url/1,
+         format_partial_url/2]).
 -export([is_absolute_URI/1]).
 -export([path_norm/1, path_norm_reverse/1,
          sanitize_file_name/1]).
@@ -343,7 +344,7 @@ parse_multi(is_end, "\r", Boundary, Acc, Res, Tmp) ->
 
 do_header([]) -> {[]};
 do_header(Head) ->
-    {ok, Fields} = regexp:split(Head, "\r\n"),
+    Fields = string:tokens(Head, "\r\n"),
     MFields = merge_lines_822(Fields),
     Header = lists:map(fun isolate_arg/1, MFields),
     case lists:keysearch("content-disposition", 1, Header) of
@@ -436,7 +437,7 @@ do_parse_spec(<<>>, Spec, Last, Cur, _State) ->
     [{Last, coerce_type(S, Cur)}];
 do_parse_spec(undefined,_,_,_,_) ->
     [];
-do_parse_spec(QueryList, Spec, Last, Cur, State) when list(QueryList) ->
+do_parse_spec(QueryList, Spec, Last, Cur, State) when is_list(QueryList) ->
     do_parse_spec(list_to_binary(QueryList), Spec, Last, Cur, State).
 
 
@@ -564,9 +565,9 @@ fl([]) ->
     [].
 
 %% htmlize
-htmlize(Bin) when binary(Bin) ->
+htmlize(Bin) when is_binary(Bin) ->
     list_to_binary(htmlize_l(binary_to_list(Bin)));
-htmlize(List) when list(List) ->
+htmlize(List) when is_list(List) ->
     htmlize_l(List).
 
 
@@ -579,8 +580,8 @@ htmlize_char($&) ->
     <<"&amp;">>;
 htmlize_char($") ->
     <<"&quot;">>;
-             htmlize_char(X) ->
-                    X.
+htmlize_char(X) ->
+    X.
 
 
 %% htmlize list (usually much more efficient than above)
@@ -596,14 +597,15 @@ htmlize_l([$&|Tail], Acc) ->
     htmlize_l(Tail, [$;,$p,$m,$a,$&|Acc]);
 htmlize_l([$"|Tail], Acc) ->
     htmlize_l(Tail, [$; , $t, $o,  $u,  $q  ,$&|Acc]);
-           htmlize_l([X|Tail], Acc) when integer(X) ->
-                  htmlize_l(Tail, [X|Acc]);
-           htmlize_l([X|Tail], Acc) when binary(X) ->
-                  X2 = htmlize_l(binary_to_list(X)),
-                  htmlize_l(Tail, [X2|Acc]);
-           htmlize_l([X|Tail], Ack) when list(X) ->
-                  X2 = htmlize_l(X),
-                  htmlize_l(Tail, [X2|Ack]).
+           
+htmlize_l([X|Tail], Acc) when is_integer(X) ->
+    htmlize_l(Tail, [X|Acc]);
+htmlize_l([X|Tail], Acc) when is_binary(X) ->
+    X2 = htmlize_l(binary_to_list(X)),
+    htmlize_l(Tail, [X2|Acc]);
+htmlize_l([X|Tail], Ack) when is_list(X) ->
+    X2 = htmlize_l(X),
+    htmlize_l(Tail, [X2|Ack]).
 
 
 
@@ -651,7 +653,7 @@ setcookie(Name, Value, Path, Expire, Domain, Secure) ->
 %% only the first match is returned
 
 
-find_cookie_val(Cookie, A) when record(A, arg) ->
+find_cookie_val(Cookie, A) when is_record(A, arg) ->
     find_cookie_val(Cookie,  (A#arg.headers)#headers.cookie);
 %%
 find_cookie_val(_Cookie, []) ->
@@ -669,7 +671,7 @@ eat_cookie([], _)           -> [];
 eat_cookie([$\s|T], Str)    -> eat_cookie(T, Str);
 eat_cookie(_, [])           -> [];
 eat_cookie(Cookie, [$\s|T]) -> eat_cookie(Cookie, T);
-eat_cookie(Cookie, Str) when list(Cookie),list(Str) ->
+eat_cookie(Cookie, Str) when is_list(Cookie),is_list(Str) ->
     try
         eat_cookie2(Cookie++"=", Str, Cookie)
     catch
@@ -701,14 +703,15 @@ url_decode([$%, Hi, Lo | Tail]) ->
             Hex = yaws:hex_to_integer([Hi, Lo]),
             [Hex | url_decode(Tail)];
             url_decode([$?|T]) ->
-                   %% Don't decode the query string here, that is parsed separately.
+                   %% Don't decode the query string here, that is 
+                   %% parsed separately.
                    [$?|T];
-            url_decode([H|T]) when integer(H) ->
+            url_decode([H|T]) when is_integer(H) ->
                    [H |url_decode(T)];
             url_decode([]) ->
                    [];
             %% deep lists
-            url_decode([H|T]) when list(H) ->
+            url_decode([H|T]) when is_list(H) ->
                    [url_decode(H) | url_decode(T)].
 
 
@@ -1070,7 +1073,7 @@ parse_url(Str, Strict) ->
         "file://" ++ Rest ->
             parse_url(host, Strict, #url{scheme = file}, Rest, []);
         _ when Strict == sloppy ->
-            parse_url(host, Strict, #url{scheme = http}, Str, [])
+            parse_url(host, Strict, #url{scheme = undefined}, Str, [])
     end.
 
 
@@ -1112,9 +1115,46 @@ parse_url(path, Strict, U, Str, Ack) ->
     end.
 
 
-format_url(Url) when record(Url, url) ->
+%% used to construct redir headers from partial URLs such 
+%% as e.g. /foo/bar
+
+format_partial_url(Url, SC) ->
+    [if 
+         Url#url.scheme == undefined ->
+             yaws:redirect_scheme(SC);
+         true ->
+             yaws:to_string(Url#url.scheme) ++ "://"
+     end,
+     if
+         Url#url.host == undefined ->
+             yaws:redirect_host(SC, undefined);
+         true ->
+             Url#url.host
+     end,
+     if
+         Url#url.port == undefined ->
+             yaws:redirect_port(SC);
+         true  ->
+             [$: | integer_to_list(Url#url.port)]
+     end,
+     Url#url.path,
+     if
+         Url#url.querypart == [] ->
+             [];
+         true ->
+             [$?|Url#url.querypart]
+     end
+    ].
+
+
+format_url(Url) when is_record(Url, url) ->
     [
-     yaws:to_string(Url#url.scheme), "://",
+     if 
+         Url#url.scheme == undefined ->
+             "http://";
+         true ->
+             yaws:to_string(Url#url.scheme) ++ "://"
+     end,
      Url#url.host,
      if
          Url#url.port == undefined ->
@@ -1159,7 +1199,7 @@ is_abs_URI1(_) ->
 %% Body  = EHTML
 
 ehtml_expand(Ch) when Ch >= 0, Ch =< 255 -> Ch; %yaws_api:htmlize_char(Ch);
-ehtml_expand(Bin) when binary(Bin) -> Bin; % yaws_api:htmlize(Bin);
+ehtml_expand(Bin) when is_binary(Bin) -> Bin; % yaws_api:htmlize(Bin);
 
 ehtml_expand({ssi,File, Del, Bs}) ->
     case yaws_server:ssi(File, Del, Bs) of
@@ -1182,7 +1222,7 @@ ehtml_expand({Tag, Attrs}) ->
     NL = ehtml_nl(Tag),
     [NL, "<", atom_to_list(Tag), ehtml_attrs(Attrs), "></",
      atom_to_list(Tag), ">"];
-ehtml_expand({Tag, Attrs, Body}) when atom(Tag) ->
+ehtml_expand({Tag, Attrs, Body}) when is_atom(Tag) ->
     Ts = atom_to_list(Tag),
     NL = ehtml_nl(Tag),
     [NL, "<", Ts, ehtml_attrs(Attrs), ">", ehtml_expand(Body), "</", Ts, ">"];
@@ -1192,27 +1232,27 @@ ehtml_expand([]) -> [].
 
 
 ehtml_attrs([]) -> [];
-ehtml_attrs([Attribute|Tail]) when atom(Attribute) ->
+ehtml_attrs([Attribute|Tail]) when is_atom(Attribute) ->
     [[$ |atom_to_list(Attribute)]|ehtml_attrs(Tail)];
-ehtml_attrs([Attribute|Tail]) when list(Attribute) ->
+ehtml_attrs([Attribute|Tail]) when is_list(Attribute) ->
     [" ", Attribute|ehtml_attrs(Tail)];
 ehtml_attrs([{Name, Value} | Tail]) ->
-    ValueString = if atom(Value) -> [$",atom_to_list(Value),$"];
-                     list(Value) -> [$",Value,$"];
-                     integer(Value) -> [$",integer_to_list(Value),$"];
-                     float(Value) -> [$",float_to_list(Value),$"]
+    ValueString = if is_atom(Value) -> [$",atom_to_list(Value),$"];
+                     is_list(Value) -> [$",Value,$"];
+                     is_integer(Value) -> [$",integer_to_list(Value),$"];
+                     is_float(Value) -> [$",float_to_list(Value),$"]
                   end,
     [[$ |atom_to_list(Name)], [$=|ValueString]|ehtml_attrs(Tail)];
 ehtml_attrs([{check, Name, Value} | Tail]) ->
-    ValueString = if atom(Value) -> [$",atom_to_list(Value),$"];
-                     list(Value) ->
+    ValueString = if is_atom(Value) -> [$",atom_to_list(Value),$"];
+                     is_list(Value) ->
                           Q = case deepmember($", Value) of
                                   true -> $';
                                   false -> $"
                               end,
                           [Q,Value,Q];
-                      integer(Value) -> [$",integer_to_list(Value),$"];
-                      float(Value) -> [$",float_to_list(Value),$"]
+                     is_integer(Value) -> [$",integer_to_list(Value),$"];
+                     is_float(Value) -> [$",float_to_list(Value),$"]
                    end,
                    [[$ |atom_to_list(Name)], 
                     [$=|ValueString]|ehtml_attrs(Tail)].
@@ -1297,7 +1337,7 @@ ehtml_expander(X) ->
 %% Text
 ehtml_expander(Ch, Before, After) when Ch >= 0, Ch =< 255 ->
     ehtml_expander_done(yaws_api:htmlize_char(Ch), Before, After);
-ehtml_expander(Bin, Before, After) when binary(Bin) ->
+ehtml_expander(Bin, Before, After) when is_binary(Bin) ->
     ehtml_expander_done(yaws_api:htmlize(Bin), Before, After);
 
 ehtml_expander({ssi,File, Del, Bs}, Before, After) ->
@@ -1327,7 +1367,7 @@ ehtml_expander({Tag, Attrs, Body}, Before, After) ->
                     Before],
                    ["</", atom_to_list(Tag), ">"|After]);
 %% Variable references
-ehtml_expander(Var, Before, After) when atom(Var) ->
+ehtml_expander(Var, Before, After) when is_atom(Var) ->
     [reverse(Before), {ehtml, ehtml_var_name(Var)}, After];
 %% Lists
 ehtml_expander([H|T], Before, After) ->
@@ -1348,17 +1388,17 @@ ehtml_attrs_expander([Var|T]) ->
     [[" ",
       ehtml_attr_part_expander(Var)]|
      ehtml_attrs_expander(T)];
-ehtml_attrs_expander(Var) when atom(Var) ->
+ehtml_attrs_expander(Var) when is_atom(Var) ->
     %% Var in the cdr of an attribute list
     [{ehtml_attrs, ehtml_var_name(Var)}].
 
-ehtml_attr_part_expander(A) when atom(A) ->
+ehtml_attr_part_expander(A) when is_atom(A) ->
     case ehtml_var_p(A) of
         true  -> {preformatted, ehtml_var_name(A)};
         false -> atom_to_list(A)
     end;
-ehtml_attr_part_expander(I) when integer(I) -> integer_to_list(I);
-ehtml_attr_part_expander(S) when list(S) -> S.
+ehtml_attr_part_expander(I) when is_integer(I) -> integer_to_list(I);
+ehtml_attr_part_expander(S) when is_list(S) -> S.
 
 ehtml_expander_done(X, Before, After) -> [reverse([X|Before]), After].
 
@@ -1366,10 +1406,10 @@ ehtml_expander_done(X, Before, After) -> [reverse([X|Before]), After].
 %% binaries.
 %% Returns: [binary() | {ehtml, Var} | {preformatted, Var}, {ehtml_attrs, Var}]
 %% Var = atom()
-ehtml_expander_compress([Tag|T], Acc) when tuple(Tag) ->
+ehtml_expander_compress([Tag|T], Acc) when is_tuple(Tag) ->
     [list_to_binary(reverse(Acc)), Tag | ehtml_expander_compress(T, [])];
 ehtml_expander_compress([], Acc) -> [list_to_binary(reverse(Acc))];
-ehtml_expander_compress([H|T], Acc) when integer(H) ->
+ehtml_expander_compress([H|T], Acc) when is_integer(H) ->
     ehtml_expander_compress(T, [H|Acc]).
 
 %% Apply an expander with the variable bindings in Env.  Env is a list of
@@ -1377,7 +1417,7 @@ ehtml_expander_compress([H|T], Acc) when integer(H) ->
 %% term.
 ehtml_apply(Expander, Env) -> [ehtml_eval(X, Env) || X <- Expander].
 
-ehtml_eval(Bin, _Env) when binary(Bin) -> Bin;
+ehtml_eval(Bin, _Env) when is_binary(Bin) -> Bin;
 ehtml_eval({Type, Var}, Env) ->
     case lists:keysearch(Var, 1, Env) of
         false -> erlang:error({ehtml_unbound, Var});
@@ -1391,14 +1431,14 @@ ehtml_eval({Type, Var}, Env) ->
 
 %% Get the name part of a variable reference.
 %% e.g. ehtml_var_name('$foo') -> foo.
-ehtml_var_name(A) when atom(A) ->
+ehtml_var_name(A) when is_atom(A) ->
     case ehtml_var_p(A) of
         true -> list_to_atom(tl(atom_to_list(A)));
         false -> erlang:error({bad_ehtml_var_name, A})
     end.
 
 %% Is X a variable reference? Variable references are atoms starting with $.
-ehtml_var_p(X) when atom(X) -> hd(atom_to_list(X)) == $$;
+ehtml_var_p(X) when is_atom(X) -> hd(atom_to_list(X)) == $$;
 ehtml_var_p(_) -> false.
 
 ehtml_expander_test() ->
@@ -1453,7 +1493,7 @@ deepmember(_C,[]) ->
     false;
 deepmember(C,[C|_Cs]) ->
     true;
-deepmember(C,[L|Cs]) when list(L) ->
+deepmember(C,[L|Cs]) when is_list(L) ->
     case deepmember(C,L) of
         true  -> true;
         false -> deepmember(C,Cs)
@@ -1604,7 +1644,7 @@ skip_space(T) -> T.
 %%
 
 
-getvar(ARG,Key) when atom(Key) ->
+getvar(ARG,Key) when is_atom(Key) ->
     getvar(ARG, atom_to_list(Key));
 getvar(ARG,Key) ->
     case (ARG#arg.req)#http_request.method of
@@ -1614,7 +1654,7 @@ getvar(ARG,Key) ->
     end.
 
 
-queryvar(ARG,Key) when atom(Key) ->
+queryvar(ARG,Key) when is_atom(Key) ->
     queryvar(ARG, atom_to_list(Key));
 queryvar(ARG, Key) ->
     Parse = case get(query_parse) of
@@ -1627,7 +1667,7 @@ queryvar(ARG, Key) ->
             end,
     filter_parse(Key, Parse).
 
-postvar(ARG, Key) when atom(Key) ->
+postvar(ARG, Key) when is_atom(Key) ->
     postvar(ARG, atom_to_list(Key));
 postvar(ARG, Key) ->
     Parse = case get(post_parse) of
@@ -1669,15 +1709,6 @@ binding_exists(Key) ->
         _ -> true
     end.
 
-%% split inputstring at first occurrence of Char
-split_at(String, Char) ->
-    split_at(String, Char, []).
-split_at([H|T], H, Ack) ->
-    {lists:reverse(Ack), T};
-split_at([H|T], Char, Ack) ->
-    split_at(T, Char, [H|Ack]);
-split_at([], _Char, Ack) ->
-    {[], lists:reverse(Ack)}.
 
 
 %% Return the parsed url that the client requested.
@@ -1686,7 +1717,7 @@ request_url(ARG) ->
     Headers = ARG#arg.headers,
     {abs_path, Path} = (ARG#arg.req)#http_request.path,
     DecPath = url_decode(Path),
-    {P,Q} = split_at(DecPath, $?),
+    {P,Q} = yaws:split_at(DecPath, $?),
     #url{scheme = case SC#sconf.ssl of
                       undefined ->
                           "http";
@@ -1730,7 +1761,7 @@ sanitize_file_name([]) ->
 
 %% to be used in embedded mode, make it possible
 %% to pass a config to yaws from another data source
-%% than /etc/yaws.conf, for example from a database
+%% than /etc/yaws/yaws.conf, for example from a database
 %% this code is also called by the server -h hup code
 setconf(GC0, Groups0) ->
     setconf(GC0, Groups0, true).
@@ -1812,7 +1843,7 @@ redirect_self(A) ->
     Host = yaws:redirect_host(get(sc), H#headers.host),
     {Scheme, SchemeStr} = 
         case {SC#sconf.ssl,SC#sconf.rmethod} of
-            {_, Method} when list(Method) ->
+            {_, Method} when is_list(Method) ->
                 {list_to_atom(Method), Method++"://"};
             {undefined,_} ->
                 {http, "http://"};
