@@ -32,11 +32,11 @@
 paths() ->
     case yaws:getuid() of
         {ok, "0"} ->    %% root 
-            [yaws_generated:etcdir() ++ "/yaws.conf"];
+            [yaws_generated:etcdir() ++ "/yaws/yaws.conf"];
         _ -> %% developer
             [filename:join([yaws:home(), "yaws.conf"]),
              "./yaws.conf", 
-             yaws_generated:etcdir() ++ "/yaws.conf"]
+             yaws_generated:etcdir() ++ "/yaws/yaws.conf"]
     end.
 
 
@@ -275,19 +275,32 @@ make_default_sconf() ->
     #sconf{docroot = filename:join([Y, "www"])}.
 
 yaws_dir() ->
-    P = filename:split(code:which(?MODULE)),
-    P1 = del_tail(P),
-    filename:join(P1).
+    case  yaws_generated:is_local_install() of
+        true ->
+            P = filename:split(code:which(?MODULE)),
+            P1 = del_tail(P),
+            filename:join(P1);
+        false ->
+            code:lib_dir(yaws)
+    end.
 
+del_tail(Parts) ->
+     del_tail(Parts,[]).
+%% Initial ".." should be preserved
+del_tail([".." |Tail], Acc) ->
+    del_tail(Tail, [".."|Acc]);
+del_tail(Parts, Acc) ->
+    del_tail2(Parts, Acc).
 
-del_tail([_H, ".." |Tail]) ->
-    del_tail(Tail);
-del_tail(["..", _H |Tail]) ->
-    del_tail(Tail);
-del_tail([_X, _Y]) ->
-    [];
-del_tail([H|T]) ->
-    [H|del_tail(T)].
+%% Embedded ".." should be removed together with preceding dir
+del_tail2([_H, ".." |Tail], Acc) ->
+    del_tail2(Tail, Acc);
+del_tail2([".." |Tail], [_P|Acc]) ->
+    del_tail2(Tail, Acc);
+del_tail2([_X, _Y], Acc) ->
+    lists:reverse(Acc);
+del_tail2([H|T], Acc) ->
+    del_tail2(T, [H|Acc]).
 
 
 
@@ -443,7 +456,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 
         ["log_wrap_size", '=', Int] ->
             case (catch list_to_integer(Int)) of
-                I when integer(I) ->
+                I when is_integer(I) ->
                     fload(FD, globals, GC#gconf{log_wrap_size = I},
                           C, Cs, Lno+1, Next);
                 _ ->
@@ -501,7 +514,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
         %% keep this bugger for backward compat for a while
         ["keepalive_timeout", '=', Val] ->
             case (catch list_to_integer(Val)) of
-                I when integer(I) ->
+                I when is_integer(I) ->
                     fload(FD, globals, GC#gconf{keepalive_timeout = I},
                           C, Cs, Lno+1, Next);
                 _ ->
@@ -523,7 +536,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 
         ["max_num_cached_files", '=', Val] ->
             case (catch list_to_integer(Val)) of
-                I when integer(I) ->
+                I when is_integer(I) ->
                     fload(FD, globals, GC#gconf{max_num_cached_files = I},
                           C, Cs, Lno+1, Next);
                 _ ->
@@ -533,7 +546,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 
         ["max_num_cached_bytes", '=', Val] ->
             case (catch list_to_integer(Val)) of
-                I when integer(I) ->
+                I when is_integer(I) ->
                     fload(FD, globals, GC#gconf{max_num_cached_bytes = I},
                           C, Cs, Lno+1, Next);
                 _ ->
@@ -543,7 +556,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 
         ["max_size_cached_file", '=', Val] ->
             case (catch list_to_integer(Val)) of
-                I when integer(I) ->
+                I when is_integer(I) ->
                     fload(FD, globals, GC#gconf{max_size_cached_file = I},
                           C, Cs, Lno+1, Next);
                 _ ->
@@ -552,7 +565,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 
         ["cache_refresh_secs", '=', Val] ->
             case (catch list_to_integer(Val)) of
-                I when integer(I), I >= 0 ->
+                I when is_integer(I), I >= 0 ->
                     fload(FD, globals, GC#gconf{cache_refresh_secs = I},
                           C, Cs, Lno+1, Next);
                 _ ->
@@ -687,7 +700,7 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
             end;
         ["port", '=', Val] ->
             case (catch list_to_integer(Val)) of
-                I when integer(I) ->
+                I when is_integer(I) ->
                     C2 = C#sconf{port = I},
                     fload(FD, server, GC, C2, Cs, Lno+1, Next);
                 _ ->
@@ -717,7 +730,7 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
             end;
         ["listen_backlog", '=', Val] ->
             case (catch list_to_integer(Val)) of
-                B when integer(B) ->
+                B when is_integer(B) ->
                     C2 = case proplists:get_value(listen_opts, C#sconf.soptions) of
                              undefined ->
                                  C#sconf{soptions = [{listen_opts, [{backlog, B}]} |
@@ -756,7 +769,7 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
                     fload(FD, server, GC, C2, Cs, Lno+1, Next);
                 Val ->
                     case (catch list_to_integer(Val)) of
-                        I when integer(I) ->
+                        I when is_integer(I) ->
                             C2 = C#sconf{partial_post_size = I},
                             fload(FD, server, GC, C2, Cs, Lno+1, Next);
                         _ ->
@@ -795,6 +808,10 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
             C2 = C#sconf{errormod_crash = list_to_atom(Module)},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
 
+	["errormod_401", '=' , Module] ->
+	    C2 = C#sconf{errormod_401 = list_to_atom(Module)},
+	    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+
         ["arg_rewrite_mod", '=', Module] ->
             C2 = C#sconf{arg_rewrite_mod = list_to_atom(Module)},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
@@ -850,6 +867,8 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
                 _URL ->
                     {error, "Can't revproxy to an URL with a path "}
             end;
+	['<', "extra_cgi_vars", "dir", '=', Dir, '>'] ->
+	    fload(FD, extra_cgi_vars, GC, C, Cs, Lno+1, Next, {Dir, []});
         [H|T] ->
             {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
     end;
@@ -866,7 +885,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
 
         ["keyfile", '=', Val] ->
             case is_file(Val) of
-                true when  record(C#sconf.ssl, ssl) ->
+                true when  is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{keyfile = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
                 true ->
@@ -877,7 +896,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
             end;
         ["certfile", '=', Val] ->
             case is_file(Val) of
-                true when  record(C#sconf.ssl, ssl) ->
+                true when  is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{certfile = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
                 true ->
@@ -888,7 +907,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
             end;
         ["cacertfile", '=', Val] ->
             case is_file(Val) of
-                true when  record(C#sconf.ssl, ssl) ->
+                true when  is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{cacertfile = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
                 true ->
@@ -900,7 +919,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
         ["verify", '=', Val0] ->
             Val = (catch list_to_integer(Val0)),
             case lists:member(Val, [1,2,3]) of
-                true when  record(C#sconf.ssl, ssl) ->
+                true when  is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{verify = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
                 true ->
@@ -912,7 +931,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
         ["depth", '=', Val0] ->
             Val = (catch list_to_integer(Val0)),
             case lists:member(Val, [1,2,3,4,5,6,7]) of
-                true when  record(C#sconf.ssl, ssl) ->
+                true when  is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{depth = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
                 true ->
@@ -923,7 +942,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
             end;
         ["password", '=', Val] ->
             if 
-                record(C#sconf.ssl, ssl) ->
+                is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{password = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
                 true ->
@@ -932,7 +951,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
             end;
         ["ciphers", '=', Val] ->
             if 
-                record(C#sconf.ssl, ssl) ->
+                is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{ciphers = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
                 true ->
@@ -1041,7 +1060,11 @@ fload(FD, server_auth, GC, C, Cs, Lno, Chars, Auth) ->
             A2 = Auth#auth{realm = Realm},
             fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
         ["authmod", '=', Mod] ->
-            A2 = Auth#auth{mod = list_to_atom(Mod)},
+	    %% Add the auth header for the mod
+	    Mod2 = list_to_atom(Mod),
+	    code:ensure_loaded(Mod2),
+	    H = Mod2:get_header() ++ Auth#auth.headers,
+            A2 = Auth#auth{mod = Mod2, headers = H},
             fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
         ["user", '=', User] ->
             case (catch string:tokens(User, ":")) of
@@ -1055,7 +1078,17 @@ fload(FD, server_auth, GC, C, Cs, Lno, Chars, Auth) ->
             A2 = Auth#auth{pam=Serv},
             fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
         ['<', "/auth", '>'] ->
-            C2 = C#sconf{authdirs = [Auth|C#sconf.authdirs]},
+	    Pam = Auth#auth.pam,
+	    Users = Auth#auth.users,
+	    Realm = Auth#auth.realm,
+	    A2 =  case {Pam, Users} of
+		    {false, []} ->
+			  Auth;
+		      _ ->
+			  H = Auth#auth.headers ++ yaws:make_www_authenticate_header({realm, Realm}),
+			  Auth#auth{headers = H}
+		  end,
+	    C2 = C#sconf{authdirs = [A2|C#sconf.authdirs]},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
         [H|T] ->
             {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
@@ -1068,25 +1101,51 @@ fload(FD, server_redirect, _GC, _C, _Cs, Lno, eof, _RedirMap) ->
 fload(FD, server_redirect, GC, C, Cs, Lno, Chars, RedirMap) ->
     %%?Debug("Chars: ~s", [Chars]),
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    Toks = toks(Chars),
+    case Toks of
         [] ->
             fload(FD, server_redirect, GC, C, Cs, Lno+1, Next, RedirMap);
-        [Path, '=', MethodHostPort] ->
-            Entry  = case MethodHostPort of
-                         "http://"++HostPort ->
-                             {Path, "http://", HostPort};
-                         "https://"++HostPort ->
-                             {Path, "https://", HostPort};
-                         HostPort ->
-                             {Path, HostPort}
-                     end,
-            fload(FD, server_redirect, GC, C, Cs, Lno+1, Next,
-                  [Entry|RedirMap]);
+        [Path, '=', URL] ->
+            io:format(" = ~p ~p~n", [URL, Toks]),
+            try yaws_api:parse_url(URL, sloppy) of
+                U when is_record(U, url) ->
+                     fload(FD, server_redirect, GC, C, Cs, Lno+1, Next,
+                           [{Path, U, append}|RedirMap])
+            catch _:_ ->
+                    {error, ?F("bad redir ~p at line ~w", [URL, Lno])}
+            end;
+        [Path, '=', '=', URL] ->
+            io:format(" == ~p~n", [URL]),
+            try yaws_api:parse_url(URL, sloppy) of
+                U when is_record(U, url) ->
+                     fload(FD, server_redirect, GC, C, Cs, Lno+1, Next,
+                           [{Path, U, noappend}|RedirMap])
+            catch _:_ ->
+                    {error, ?F("Bad redir ~p at line ~w", [URL, Lno])}
+            end;
         ['<', "/redirect", '>'] ->
             C2 = C#sconf{redirect_map = lists:reverse(RedirMap)},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
         [H|T] ->
             {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+    end;
+
+fload(FD, extra_cgi_vars, _GC, _C, _Cs, Lno, eof, _EVars) ->
+    file:close(FD),
+    {error, ?F("Unexpected end of file at line ~w", [Lno])};
+
+fload(FD, extra_cgi_vars, GC, C, Cs, Lno, Chars, EVars = {Dir, Vars}) ->
+    Next = io:get_line(FD, ''),
+    case toks(Chars) of
+        [] ->
+            fload(FD, extra_cgi_vars, GC, C, Cs, Lno+1, Next, EVars);
+        [Var, '=', Val] ->
+	    fload(FD, extra_cgi_vars, GC, C, Cs, Lno+1, Next, {Dir, [{Var, Val} | Vars]});
+        ['<', "/extra_cgi_vars", '>'] ->
+            C2 = C#sconf{extra_cgi_vars = [EVars | C#sconf.extra_cgi_vars]},
+            fload(FD, server, GC, C2, Cs, Lno+1, Next);
+        [H|T] ->
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
     end.
 
 
@@ -1504,7 +1563,7 @@ find_sc(_SC,[]) ->
 
 
 
-verify_upgrade_args(GC, Groups0) when record(GC, gconf) ->
+verify_upgrade_args(GC, Groups0) when is_record(GC, gconf) ->
     case is_groups(Groups0) of
         true ->
             %% embeded code may give appmods as a list of strings
@@ -1516,9 +1575,9 @@ verify_upgrade_args(GC, Groups0) when record(GC, gconf) ->
                                         lists:map(
                                           fun({PE, Mod}) ->
                                                   {PE, Mod};
-                                             (AM) when list(AM) ->
+                                             (AM) when is_list(AM) ->
                                                   {AM,list_to_atom(AM)};
-                                             (AM) when atom(AM) ->
+                                             (AM) when is_atom(AM) ->
                                                   {atom_to_list(AM), AM}
                                           end,
                                           SC#sconf.appmods)}
@@ -1540,7 +1599,7 @@ is_groups([H|T]) ->
 is_groups([]) ->
     true.
 
-is_list_of_scs([H|T]) when record(H, sconf) ->
+is_list_of_scs([H|T]) when is_record(H, sconf) ->
     is_list_of_scs(T);
 is_list_of_scs([]) ->
     true;
