@@ -13,6 +13,7 @@
 	 initModelFile/1,
 	 config_file_xsd/0,
 	 call/3, call/4, call/6,
+	 call_attach/4, call_attach/5, call_attach/7,
 	 write_hrl/2, write_hrl/3,
 	 findHeader/2,
 	 parseMessage/2,
@@ -33,6 +34,8 @@
 -include("../include/erlsom.hrl").  % FIXME a better solution ?
 
 
+-define(HTTP_REQ_TIMEOUT, 20000).
+
 %%-define(dbg(X,Y), 
 %%        error_logger:info_msg("*dbg ~p(~p): " X,
 %%                              [?MODULE, ?LINE | Y])).
@@ -53,12 +56,12 @@
 %%%
 %%% Writes the header file (record definitions) for a WSDL file
 %%%
-write_hrl(WsdlURL, Output) when list(WsdlURL) ->
+write_hrl(WsdlURL, Output) when is_list(WsdlURL) ->
     write_hrl(initModel(WsdlURL), Output);
-write_hrl(#wsdl{model = Model}, Output) when list(Output) ->
+write_hrl(#wsdl{model = Model}, Output) when is_list(Output) ->
     erlsom:write_hrl(Model, Output).
 
-write_hrl(WsdlURL, Output, Prefix) when list(WsdlURL),list(Prefix) ->
+write_hrl(WsdlURL, Output, Prefix) when is_list(WsdlURL),is_list(Prefix) ->
     write_hrl(initModel(WsdlURL, Prefix), Output).
 
 
@@ -72,7 +75,7 @@ qtest() ->
 %%% --------------------------------------------------------------------
 %%% Access functions
 %%% --------------------------------------------------------------------
-is_wsdl(Wsdl) when record(Wsdl,wsdl) -> true;
+is_wsdl(Wsdl) when is_record(Wsdl,wsdl) -> true;
 is_wsdl(_)                           -> false.
 
 wsdl_operations(#wsdl{operations = Ops}) -> Ops.
@@ -95,14 +98,15 @@ wsdl_op_action(#operation{action = Action}) -> Action.
 %%% --------------------------------------------------------------------
 %%% For Quick deployment
 %%% --------------------------------------------------------------------
-call(WsdlURL, Operation, ListOfData) when list(WsdlURL) ->
+call(WsdlURL, Operation, ListOfData) when is_list(WsdlURL) ->
     Wsdl = initModel(WsdlURL, ?DefaultPrefix),
     call(Wsdl, Operation, ListOfData);
-call(Wsdl, Operation, ListOfData) when record(Wsdl, wsdl) ->
+call(Wsdl, Operation, ListOfData) when is_record(Wsdl, wsdl) ->
     case get_operation(Wsdl#wsdl.operations, Operation) of
 	{ok, Op} ->
 	    Msg = mk_msg(?DefaultPrefix, Operation, ListOfData),
-	    call(Wsdl, Operation, Op#operation.port, Op#operation.service, [], Msg);
+	    call(Wsdl, Operation, Op#operation.port, 
+                 Op#operation.service, [], Msg);
 	Else ->
 	    Else
     end.
@@ -110,10 +114,10 @@ call(Wsdl, Operation, ListOfData) when record(Wsdl, wsdl) ->
 %%% --------------------------------------------------------------------
 %%% Takes the actual records for the Header and Body message.
 %%% --------------------------------------------------------------------
-call(WsdlURL, Operation, Header, Msg) when list(WsdlURL) ->
+call(WsdlURL, Operation, Header, Msg) when is_list(WsdlURL) ->
     Wsdl = initModel(WsdlURL, ?DefaultPrefix),
     call(Wsdl, Operation, Header, Msg);
-call(Wsdl, Operation, Header, Msg) when record(Wsdl, wsdl) ->
+call(Wsdl, Operation, Header, Msg) when is_record(Wsdl, wsdl) ->
     case get_operation(Wsdl#wsdl.operations, Operation) of
 	{ok, Op} ->
 	    call(Wsdl, Operation, Op#operation.port, Op#operation.service, 
@@ -128,15 +132,64 @@ mk_msg(Prefix, Operation, ListOfData) ->
 		   []                                    % anyAttribs
 		   | ListOfData]).                       % rest of record data
 
-get_operation([#operation{operation = X} = Op|_], X) -> {ok, Op};
-get_operation([_|T], Op)                             -> get_operation(T, Op);
-get_operation([], _Op)                               -> {error, "operation not found"}.
+get_operation([#operation{operation = X} = Op|_], X) -> 
+    {ok, Op};
+get_operation([_|T], Op)                             -> 
+    get_operation(T, Op);
+get_operation([], _Op)                               -> 
+    {error, "operation not found"}.
 
 
 %%% --------------------------------------------------------------------
-%%% Make a SOAP request
+%%% Make a SOAP request (no attachments)
 %%% --------------------------------------------------------------------
-call(#wsdl{operations = Operations, model = Model}, Operation, Port, Service, Headers, Message) ->
+call(Wsdl, Operation, Port, Service, Headers, Message) ->
+	call_attach(Wsdl, Operation, Port, Service, Headers, Message, []).
+
+
+%%% --------------------------------------------------------------------
+%%% For Quick deployment (with attachments)
+%%% --------------------------------------------------------------------
+call_attach(WsdlURL, Operation, ListOfData, Attachments) 
+  when is_list(WsdlURL) ->
+    Wsdl = initModel(WsdlURL, ?DefaultPrefix),
+    call_attach(Wsdl, Operation, ListOfData, Attachments);
+call_attach(Wsdl, Operation, ListOfData, Attachments) 
+  when is_record(Wsdl, wsdl) ->
+    case get_operation(Wsdl#wsdl.operations, Operation) of
+	{ok, Op} ->
+	    Msg = mk_msg(?DefaultPrefix, Operation, ListOfData),
+	    call_attach(Wsdl, Operation, Op#operation.port, 
+                        Op#operation.service, [], Msg, Attachments);
+	Else ->
+	    Else
+    end.
+
+%%% --------------------------------------------------------------------
+%%% Takes the actual records for the Header and Body message 
+%%% (with attachments)
+%%% --------------------------------------------------------------------
+call_attach(WsdlURL, Operation, Header, Msg, Attachments) 
+  when is_list(WsdlURL) ->
+    Wsdl = initModel(WsdlURL, ?DefaultPrefix),
+    call_attach(Wsdl, Operation, Header, Msg, Attachments);
+call_attach(Wsdl, Operation, Header, Msg, Attachments) 
+  when is_record(Wsdl, wsdl) ->
+    case get_operation(Wsdl#wsdl.operations, Operation) of
+	{ok, Op} ->
+	    call_attach(Wsdl, Operation, Op#operation.port, 
+                        Op#operation.service, 
+		 Header, Msg, Attachments);
+	Else ->
+	    Else
+    end.
+    
+    
+%%% --------------------------------------------------------------------
+%%% Make a SOAP request (with attachments)
+%%% --------------------------------------------------------------------
+call_attach(#wsdl{operations = Operations, model = Model}, 
+            Operation, Port, Service, Headers, Message, Attachments) ->
     %% find the operation
     case findOperation(Operation, Port, Service, Operations) of
 	#operation{address = URL, action = SoapAction} ->
@@ -145,18 +198,22 @@ call(#wsdl{operations = Operations, model = Model}, Operation, Port, Service, He
 	    %% Encode the message
 	    case erlsom:write(Envelope, Model) of
 		{ok, XmlMessage} ->
-		    Request = make_request_body(XmlMessage),
+				
+		    {ContentType, Request} = 
+                        make_request_body(XmlMessage, Attachments),
 		    HttpHeaders = [],
 		    HttpClientOptions = [],
                     ?dbg("+++ Request = ~p~n", [Request]),
 		    HttpRes = http_request(URL, SoapAction, Request, 
-                                           HttpClientOptions, HttpHeaders),
+                                           HttpClientOptions, HttpHeaders, 
+                                           ContentType),
                     ?dbg("+++ HttpRes = ~p~n", [HttpRes]),
 		    case HttpRes of
 			{ok, _Code, _ReturnHeaders, Body} ->
 			    parseMessage(Body, Model);
 			Error -> 
-			    %% in case of HTTP error: return {error, description}
+			    %% in case of HTTP error: return 
+                            %% {error, description}
 			    Error
 		    end;
 		{error, EncodingError} ->
@@ -195,12 +252,12 @@ findOperation(Operation, Port, Service, [#operation{} | Tail]) ->
     findOperation(Operation, Port, Service, Tail).
 
 
-mk_envelope(M, H) when tuple(M) -> mk_envelope([M], H);
-mk_envelope(M, H) when tuple(H) -> mk_envelope(M, [H]);
+mk_envelope(M, H) when is_tuple(M) -> mk_envelope([M], H);
+mk_envelope(M, H) when is_tuple(H) -> mk_envelope(M, [H]);
 %%
-mk_envelope(Messages, []) when list(Messages) ->
+mk_envelope(Messages, []) when is_list(Messages) ->
     #'soap:Envelope'{'Body' =  #'soap:Body'{choice = Messages}};
-mk_envelope(Messages, Headers) when list(Messages),list(Headers) ->
+mk_envelope(Messages, Headers) when is_list(Messages),is_list(Headers) ->
     #'soap:Envelope'{'Body'   =  #'soap:Body'{choice   = Messages},
 		     'Header' =  #'soap:Header'{choice = Headers}}.
 
@@ -334,25 +391,25 @@ get_url_file(Fname) ->
 %%% --------------------------------------------------------------------
 %%% Make a HTTP Request
 %%% --------------------------------------------------------------------
-http_request(URL, SoapAction, Request, Options, Headers) ->
+http_request(URL, SoapAction, Request, Options, Headers, ContentType) ->
     case code:ensure_loaded(ibrowse) of
 	{module, ibrowse} ->
 	    %% If ibrowse exist in the path then let's use it...
-	    ibrowse_request(URL, SoapAction, Request, Options, Headers);
+	    ibrowse_request(URL, SoapAction, Request, Options, Headers, ContentType);
 	_ ->
 	    %% ...otherwise, let's use the OTP http client.
-	    inets_request(URL, SoapAction, Request, Options, Headers)
+	    inets_request(URL, SoapAction, Request, Options, Headers, ContentType)
     end.
 
-inets_request(URL, SoapAction, Request, Options, Headers) ->
+inets_request(URL, SoapAction, Request, Options, Headers, ContentType) ->
     NewHeaders = [{"Host", "localhost:8800"}, {"SOAPAction", SoapAction}|Headers],
     NewOptions = [{cookies, enabled}|Options],
     http:set_options(NewOptions),
     case http:request(post,
                       {URL,NewHeaders,
-                       "text/xml; charset=utf-8",
+                       ContentType,
                        Request},
-                      [{timeout,20000}],
+                      [{timeout,?HTTP_REQ_TIMEOUT}],
                       [{sync, true}, {full_result, true}, {body_format, string}]) of
         {ok,{{_HTTP,200,_OK},ResponseHeaders,ResponseBody}} ->
             {ok, 200, ResponseHeaders, ResponseBody};
@@ -364,10 +421,10 @@ inets_request(URL, SoapAction, Request, Options, Headers) ->
             Other
     end.
 
-ibrowse_request(URL, SoapAction, Request, Options, Headers) ->
+ibrowse_request(URL, SoapAction, Request, Options, Headers, ContentType) ->
     case start_ibrowse() of
         ok ->
-            NewHeaders = [{"Content-Type", "text/xml; charset=utf-8"}, {"SOAPAction", SoapAction} | Headers],
+            NewHeaders = [{"Content-Type", ContentType}, {"SOAPAction", SoapAction} | Headers],
             NewOptions = Options,
                          %%[{content_type, "text/xml; encoding=utf-8"} | Options],
             case ibrowse:send_req(URL, NewHeaders, post, Request, NewOptions) of
@@ -391,8 +448,10 @@ start_ibrowse() ->
 rmsp(Str) -> string:strip(Str, left).
 
 
-make_request_body(Content) ->
-        "<?xml version=\"1.0\" encoding=\"utf-8\"?>"++Content.
+make_request_body(Content, []) ->
+        {"text/xml; charset=utf-8", "<?xml version=\"1.0\" encoding=\"utf-8\"?>"++Content};
+make_request_body(Content, AttachedFiles) ->
+		{"application/dime", yaws_dime:encode("<?xml version=\"1.0\" encoding=\"utf-8\"?>"++Content, AttachedFiles)}.
 
 
 makeFault(FaultCode, FaultString) ->
